@@ -3,7 +3,7 @@ Basic skeleton of a mitmproxy addon.
 
 Run as follows: mitmproxy -s anatomy.py
 """
-from mitmproxy import ctx
+import logging
 
 
 class Counter:
@@ -12,7 +12,7 @@ class Counter:
 
     def request(self, flow):
         self.num = self.num + 1
-        ctx.log.info("We've seen %d flows" % self.num)
+        logging.info("We've seen %d flows" % self.num)
 
 
 addons = [Counter()]
