@@ -1,11 +1,12 @@
 """Handle file paths as command arguments."""
+import logging
 from collections.abc import Sequence
 
 from mitmproxy import command
-from mitmproxy import ctx
 from mitmproxy import flow
 from mitmproxy import http
 from mitmproxy import types
+from mitmproxy.log import ALERT
 
 
 class MyAddon:
@@ -24,7 +25,7 @@ class MyAddon:
             for cnt, dom in sorted((v, k) for (k, v) in totals.items()):
                 fp.write(f"{cnt}: {dom}\n")
 
-        ctx.log.alert("done")
+        logging.log(ALERT, "done")
 
 
 addons = [MyAddon()]
