@@ -1,6 +1,7 @@
 """Add a custom command to mitmproxy's command prompt."""
+import logging
+
 from mitmproxy import command
-from mitmproxy import ctx
 
 
 class MyAddon:
@@ -10,7 +11,7 @@ class MyAddon:
     @command.command("myaddon.inc")
     def inc(self) -> None:
         self.num += 1
-        ctx.log.info(f"num = {self.num}")
+        logging.info(f"num = {self.num}")
 
 
 addons = [MyAddon()]
