@@ -3,7 +3,7 @@ Add a new mitmproxy option.
 
 Usage:
 
-    mitmproxy -s options-simple.py --set addheader true
+    mitmproxy -s options-simple.py --set addheader=true
 """
 from mitmproxy import ctx
 
