@@ -10,15 +10,16 @@ Example Invocation:
 
     mitmdump --rawtcp --tcp-hosts ".*" -s examples/tcp-simple.py
 """
-from mitmproxy.utils import strutils
-from mitmproxy import ctx
+import logging
+
 from mitmproxy import tcp
+from mitmproxy.utils import strutils
 
 
 def tcp_message(flow: tcp.TCPFlow):
     message = flow.messages[-1]
     message.content = message.content.replace(b"foo", b"bar")
 
-    ctx.log.info(
+    logging.info(
         f"tcp_message[from_client={message.from_client}), content={strutils.bytes_to_escaped_str(message.content)}]"
     )
