@@ -5,7 +5,8 @@ This example shows how to inject a WebSocket message into a running connection.
 """
 import asyncio
 
-from mitmproxy import ctx, http
+from mitmproxy import ctx
+from mitmproxy import http
 
 
 # Simple example: Inject a message as a response to an event
@@ -33,5 +34,12 @@ async def inject_async(flow: http.HTTPFlow):
         msg = msg[1:] + msg[:1]
 
 
+# Python 3.11: replace with TaskGroup
+tasks = set()
+
+
 def websocket_start(flow: http.HTTPFlow):
-    asyncio.create_task(inject_async(flow))
+    # we need to hold a reference to the task, otherwise it will be garbage collected.
+    t = asyncio.create_task(inject_async(flow))
+    tasks.add(t)
+    t.add_done_callback(tasks.remove)
