@@ -30,10 +30,13 @@ Configuration:
         dump_destination: "/user/rastley/output.log"
         EOF
 """
-from threading import Lock, Thread
-from queue import Queue
 import base64
 import json
+import logging
+from queue import Queue
+from threading import Lock
+from threading import Thread
+
 import requests
 
 from mitmproxy import ctx
@@ -48,6 +51,7 @@ class JSONDumper:
     for out-of-the-box Elasticsearch support, and then either writes
     the result to a file or sends it to a URL.
     """
+
     def __init__(self):
         self.outfile = None
         self.transformations = None
@@ -63,76 +67,77 @@ class JSONDumper:
             self.outfile.close()
 
     fields = {
-        'timestamp': (
-            ('error', 'timestamp'),
-
-            ('request', 'timestamp_start'),
-            ('request', 'timestamp_end'),
-
-            ('response', 'timestamp_start'),
-            ('response', 'timestamp_end'),
-
-            ('client_conn', 'timestamp_start'),
-            ('client_conn', 'timestamp_end'),
-            ('client_conn', 'timestamp_tls_setup'),
-
-            ('server_conn', 'timestamp_start'),
-            ('server_conn', 'timestamp_end'),
-            ('server_conn', 'timestamp_tls_setup'),
-            ('server_conn', 'timestamp_tcp_setup'),
+        "timestamp": (
+            ("error", "timestamp"),
+            ("request", "timestamp_start"),
+            ("request", "timestamp_end"),
+            ("response", "timestamp_start"),
+            ("response", "timestamp_end"),
+            ("client_conn", "timestamp_start"),
+            ("client_conn", "timestamp_end"),
+            ("client_conn", "timestamp_tls_setup"),
+            ("server_conn", "timestamp_start"),
+            ("server_conn", "timestamp_end"),
+            ("server_conn", "timestamp_tls_setup"),
+            ("server_conn", "timestamp_tcp_setup"),
         ),
-        'ip': (
-            ('server_conn', 'source_address'),
-            ('server_conn', 'ip_address'),
-            ('server_conn', 'address'),
-            ('client_conn', 'address'),
+        "ip": (
+            ("server_conn", "source_address"),
+            ("server_conn", "ip_address"),
+            ("server_conn", "address"),
+            ("client_conn", "address"),
         ),
-        'ws_messages': (
-            ('messages', ),
+        "ws_messages": (("messages",),),
+        "headers": (
+            ("request", "headers"),
+            ("response", "headers"),
         ),
-        'headers': (
-            ('request', 'headers'),
-            ('response', 'headers'),
-        ),
-        'content': (
-            ('request', 'content'),
-            ('response', 'content'),
+        "content": (
+            ("request", "content"),
+            ("response", "content"),
         ),
     }
 
     def _init_transformations(self):
         self.transformations = [
             {
-                'fields': self.fields['headers'],
-                'func': dict,
+                "fields": self.fields["headers"],
+                "func": dict,
             },
             {
-                'fields': self.fields['timestamp'],
-                'func': lambda t: int(t * 1000),
+                "fields": self.fields["timestamp"],
+                "func": lambda t: int(t * 1000),
             },
             {
-                'fields': self.fields['ip'],
-                'func': lambda addr: {
-                    'host': addr[0].replace('::ffff:', ''),
-                    'port': addr[1],
+                "fields": self.fields["ip"],
+                "func": lambda addr: {
+                    "host": addr[0].replace("::ffff:", ""),
+                    "port": addr[1],
                 },
             },
             {
-                'fields': self.fields['ws_messages'],
-                'func': lambda ms: [{
-                    'type': m[0],
-                    'from_client': m[1],
-                    'content': base64.b64encode(bytes(m[2], 'utf-8')) if self.encode else m[2],
-                    'timestamp': int(m[3] * 1000),
-                } for m in ms],
-            }
+                "fields": self.fields["ws_messages"],
+                "func": lambda ms: [
+                    {
+                        "type": m[0],
+                        "from_client": m[1],
+                        "content": base64.b64encode(bytes(m[2], "utf-8"))
+                        if self.encode
+                        else m[2],
+                        "timestamp": int(m[3] * 1000),
+                    }
+                    for m in ms
+                ],
+            },
         ]
 
         if self.encode:
-            self.transformations.append({
-                'fields': self.fields['content'],
-                'func': base64.b64encode,
-            })
+            self.transformations.append(
+                {
+                    "fields": self.fields["content"],
+                    "func": base64.b64encode,
+                }
+            )
 
     @staticmethod
     def transform_field(obj, path, func):
@@ -153,8 +158,10 @@ class JSONDumper:
         Recursively convert all list/dict elements of type `bytes` into strings.
         """
         if isinstance(obj, dict):
-            return {cls.convert_to_strings(key): cls.convert_to_strings(value)
-                    for key, value in obj.items()}
+            return {
+                cls.convert_to_strings(key): cls.convert_to_strings(value)
+                for key, value in obj.items()
+            }
         elif isinstance(obj, list) or isinstance(obj, tuple):
             return [cls.convert_to_strings(element) for element in obj]
         elif isinstance(obj, bytes):
@@ -172,8 +179,8 @@ class JSONDumper:
         Transform and dump (write / send) a data frame.
         """
         for tfm in self.transformations:
-            for field in tfm['fields']:
-                self.transform_field(frame, field, tfm['func'])
+            for field in tfm["fields"]:
+                self.transform_field(frame, field, tfm["func"])
         frame = self.convert_to_strings(frame)
 
         if self.outfile:
@@ -188,14 +195,21 @@ class JSONDumper:
         """
         Extra options to be specified in `~/.mitmproxy/config.yaml`.
         """
-        loader.add_option('dump_encodecontent', bool, False,
-                          'Encode content as base64.')
-        loader.add_option('dump_destination', str, 'jsondump.out',
-                          'Output destination: path to a file or URL.')
-        loader.add_option('dump_username', str, '',
-                          'Basic auth username for URL destinations.')
-        loader.add_option('dump_password', str, '',
-                          'Basic auth password for URL destinations.')
+        loader.add_option(
+            "dump_encodecontent", bool, False, "Encode content as base64."
+        )
+        loader.add_option(
+            "dump_destination",
+            str,
+            "jsondump.out",
+            "Output destination: path to a file or URL.",
+        )
+        loader.add_option(
+            "dump_username", str, "", "Basic auth username for URL destinations."
+        )
+        loader.add_option(
+            "dump_password", str, "", "Basic auth password for URL destinations."
+        )
 
     def configure(self, _):
         """
@@ -204,18 +218,18 @@ class JSONDumper:
         """
         self.encode = ctx.options.dump_encodecontent
 
-        if ctx.options.dump_destination.startswith('http'):
+        if ctx.options.dump_destination.startswith("http"):
             self.outfile = None
             self.url = ctx.options.dump_destination
-            ctx.log.info('Sending all data frames to %s' % self.url)
+            logging.info("Sending all data frames to %s" % self.url)
             if ctx.options.dump_username and ctx.options.dump_password:
                 self.auth = (ctx.options.dump_username, ctx.options.dump_password)
-                ctx.log.info('HTTP Basic auth enabled.')
+                logging.info("HTTP Basic auth enabled.")
         else:
-            self.outfile = open(ctx.options.dump_destination, 'a')
+            self.outfile = open(ctx.options.dump_destination, "a")
             self.url = None
             self.lock = Lock()
-            ctx.log.info('Writing all data frames to %s' % ctx.options.dump_destination)
+            logging.info("Writing all data frames to %s" % ctx.options.dump_destination)
 
         self._init_transformations()
 
