@@ -1,17 +1,20 @@
 import contextlib
 import inspect
+import logging
 import pprint
 import sys
 import traceback
 import types
-from collections.abc import Callable, Sequence
+from collections.abc import Callable
+from collections.abc import Sequence
 from dataclasses import dataclass
-from typing import Any, Optional
+from typing import Any
 
-from mitmproxy import hooks
 from mitmproxy import exceptions
 from mitmproxy import flow
-from . import ctx
+from mitmproxy import hooks
+
+logger = logging.getLogger(__name__)
 
 
 def _get_name(itm):
@@ -48,8 +51,11 @@ def safecall():
         etype, value, tb = sys.exc_info()
         tb = cut_traceback(tb, "invoke_addon_sync")
         tb = cut_traceback(tb, "invoke_addon")
-        ctx.log.error(
-            "Addon error: %s" % "".join(traceback.format_exception(etype, value, tb))
+        assert etype
+        assert value
+        logger.error(
+            f"Addon error: {value}",
+            exc_info=(etype, value, tb),
         )
 
 
@@ -67,7 +73,7 @@ class Loader:
         typespec: type,
         default: Any,
         help: str,
-        choices: Optional[Sequence[str]] = None,
+        choices: Sequence[str] | None = None,
     ) -> None:
         """
         Add an option to mitmproxy.
@@ -88,7 +94,7 @@ class Loader:
             if same_signature:
                 return
             else:
-                ctx.log.warn("Over-riding existing option %s" % name)
+                logger.warning("Over-riding existing option %s" % name)
         self.master.options.add_option(name, typespec, default, help, choices)
 
     def add_command(self, path: str, func: Callable) -> None:
@@ -128,7 +134,7 @@ class AddonManager:
         self.master = master
         master.options.changed.connect(self._configure_all)
 
-    def _configure_all(self, options, updated):
+    def _configure_all(self, updated):
         self.trigger(hooks.ConfigureHook(updated))
 
     def clear(self):
@@ -160,17 +166,18 @@ class AddonManager:
         """
         api_changes = {
             # mitmproxy 6 -> mitmproxy 7
-            "clientconnect": "client_connected",
-            "clientdisconnect": "client_disconnected",
-            "serverconnect": "server_connect and server_connected",
-            "serverdisconnect": "server_disconnected",
+            "clientconnect": f"The clientconnect event has been removed, use client_connected instead",
+            "clientdisconnect": f"The clientdisconnect event has been removed, use client_disconnected instead",
+            "serverconnect": "The serverconnect event has been removed, use server_connect and server_connected instead",
+            "serverdisconnect": f"The serverdisconnect event has been removed, use server_disconnected instead",
+            # mitmproxy 8 -> mitmproxy 9
+            "add_log": "The add_log event has been deprecated, use Python's builtin logging module instead",
         }
         for a in traverse([addon]):
-            for old, new in api_changes.items():
+            for old, msg in api_changes.items():
                 if hasattr(a, old):
-                    ctx.log.warn(
-                        f"The {old} event has been removed, use {new} instead. "
-                        f"For more details, see https://docs.mitmproxy.org/stable/addons-events/."
+                    logger.warning(
+                        f"{msg}. For more details, see https://docs.mitmproxy.org/dev/addons-api-changelog/."
                     )
             name = _get_name(a)
             if name in self.lookup:
