@@ -12,46 +12,54 @@ metadata depend on the protocol in use. Known attributes can be found in
 `base.View`.
 """
 import traceback
-from typing import Union
-from typing import Optional
-
-import blinker
 
+from . import auto
+from . import css
+from . import graphql
+from . import grpc
+from . import hex
+from . import http3
+from . import image
+from . import javascript
+from . import json
+from . import mqtt
+from . import msgpack
+from . import multipart
+from . import protobuf
+from . import query
+from . import raw
+from . import urlencoded
+from . import wbxml
+from . import xml_html
+from ..tcp import TCPMessage
+from ..udp import UDPMessage
+from ..websocket import WebSocketMessage
+from .base import format_dict
+from .base import format_text
+from .base import KEY_MAX
+from .base import TViewResult
+from .base import View
 from mitmproxy import flow
 from mitmproxy import http
+from mitmproxy import tcp
+from mitmproxy import udp
+from mitmproxy.utils import signals
 from mitmproxy.utils import strutils
-from . import (
-    auto,
-    raw,
-    hex,
-    json,
-    xml_html,
-    wbxml,
-    javascript,
-    css,
-    urlencoded,
-    multipart,
-    image,
-    query,
-    protobuf,
-    msgpack,
-    graphql,
-    grpc,
-)
-from .base import View, KEY_MAX, format_text, format_dict, TViewResult
-from ..http import HTTPFlow
-from ..tcp import TCPMessage, TCPFlow
-from ..websocket import WebSocketMessage
 
 views: list[View] = []
 
-on_add = blinker.Signal()
+
+def _update(view: View) -> None:
+    ...
+
+
+on_add = signals.SyncSignal(_update)
 """A new contentview has been added."""
-on_remove = blinker.Signal()
+on_remove = signals.SyncSignal(_update)
 """A contentview has been removed."""
 
 
-def get(name: str) -> Optional[View]:
+def get(name: str) -> View | None:
     for i in views:
         if i.name.lower() == name.lower():
             return i
@@ -79,7 +87,7 @@ def safe_to_print(lines, encoding="utf8"):
     """
     for line in lines:
         clean_line = []
-        for (style, text) in line:
+        for style, text in line:
             if isinstance(text, bytes):
                 text = text.decode(encoding, "replace")
             text = strutils.escape_control_characters(text)
@@ -89,8 +97,8 @@ def safe_to_print(lines, encoding="utf8"):
 
 def get_message_content_view(
     viewname: str,
-    message: Union[http.Message, TCPMessage, WebSocketMessage],
-    flow: Union[HTTPFlow, TCPFlow],
+    message: http.Message | TCPMessage | UDPMessage | WebSocketMessage,
+    flow: flow.Flow,
 ):
     """
     Like get_content_view, but also handles message encoding.
@@ -100,7 +108,7 @@ def get_message_content_view(
         viewmode = get("auto")
     assert viewmode
 
-    content: Optional[bytes]
+    content: bytes | None
     try:
         content = message.content
     except ValueError:
@@ -124,12 +132,22 @@ def get_message_content_view(
             if ct := http.parse_content_type(ctype):
                 content_type = f"{ct[0]}/{ct[1]}"
 
+    tcp_message = None
+    if isinstance(message, TCPMessage):
+        tcp_message = message
+
+    udp_message = None
+    if isinstance(message, UDPMessage):
+        udp_message = message
+
     description, lines, error = get_content_view(
         viewmode,
         content,
         content_type=content_type,
         flow=flow,
         http_message=http_message,
+        tcp_message=tcp_message,
+        udp_message=udp_message,
     )
 
     if enc:
@@ -138,30 +156,15 @@ def get_message_content_view(
     return description, lines, error
 
 
-def get_tcp_content_view(
-    viewname: str,
-    data: bytes,
-    flow: TCPFlow,
-):
-    viewmode = get(viewname)
-    if not viewmode:
-        viewmode = get("auto")
-
-    # https://github.com/mitmproxy/mitmproxy/pull/3970#issuecomment-623024447
-    assert viewmode
-
-    description, lines, error = get_content_view(viewmode, data, flow=flow)
-
-    return description, lines, error
-
-
 def get_content_view(
     viewmode: View,
     data: bytes,
     *,
-    content_type: Optional[str] = None,
-    flow: Optional[flow.Flow] = None,
-    http_message: Optional[http.Message] = None,
+    content_type: str | None = None,
+    flow: flow.Flow | None = None,
+    http_message: http.Message | None = None,
+    tcp_message: tcp.TCPMessage | None = None,
+    udp_message: udp.UDPMessage | None = None,
 ):
     """
     Args:
@@ -176,7 +179,12 @@ def get_content_view(
     """
     try:
         ret = viewmode(
-            data, content_type=content_type, flow=flow, http_message=http_message
+            data,
+            content_type=content_type,
+            flow=flow,
+            http_message=http_message,
+            tcp_message=tcp_message,
+            udp_message=udp_message,
         )
         if ret is None:
             ret = (
@@ -186,6 +194,8 @@ def get_content_view(
                     content_type=content_type,
                     flow=flow,
                     http_message=http_message,
+                    tcp_message=tcp_message,
+                    udp_message=udp_message,
                 )[1],
             )
         desc, content = ret
@@ -196,7 +206,12 @@ def get_content_view(
         raw = get("Raw")
         assert raw
         content = raw(
-            data, content_type=content_type, flow=flow, http_message=http_message
+            data,
+            content_type=content_type,
+            flow=flow,
+            http_message=http_message,
+            tcp_message=tcp_message,
+            udp_message=udp_message,
         )[1]
         error = f"{getattr(viewmode, 'name')} content viewer failed: \n{traceback.format_exc()}"
 
@@ -220,6 +235,8 @@ add(query.ViewQuery())
 add(protobuf.ViewProtobuf())
 add(msgpack.ViewMsgPack())
 add(grpc.ViewGrpcProtobuf())
+add(mqtt.ViewMQTT())
+add(http3.ViewHttp3())
 
 __all__ = [
     "View",
