@@ -1,5 +1,9 @@
 import os
-from typing import Any, BinaryIO, Iterable, Union, cast
+from collections.abc import Iterable
+from typing import Any
+from typing import BinaryIO
+from typing import cast
+from typing import Union
 
 from mitmproxy import exceptions
 from mitmproxy import flow
@@ -35,11 +39,11 @@ class FlowReader:
                 try:
                     yield flow.Flow.from_state(compat.migrate_flow(loaded))
                 except ValueError as e:
-                    raise exceptions.FlowReadException(e)
+                    raise exceptions.FlowReadException(e) from e
         except (ValueError, TypeError, IndexError) as e:
             if str(e) == "not a tnetstring: empty file":
                 return  # Error is due to EOF
-            raise exceptions.FlowReadException("Invalid data format.")
+            raise exceptions.FlowReadException("Invalid data format.") from e
 
 
 class FilteredFlowWriter:
