@@ -1,17 +1,22 @@
 from mitmproxy import addons
 from mitmproxy import master
 from mitmproxy import options
-from mitmproxy.addons import dumper, errorcheck, keepserving, readfile, termlog
+from mitmproxy.addons import dumper
+from mitmproxy.addons import errorcheck
+from mitmproxy.addons import keepserving
+from mitmproxy.addons import readfile
+from mitmproxy.addons import termlog
 
 
 class DumpMaster(master.Master):
     def __init__(
         self,
         options: options.Options,
+        loop=None,
         with_termlog=True,
         with_dumper=True,
     ) -> None:
-        super().__init__(options)
+        super().__init__(options, event_loop=loop)
         if with_termlog:
             self.addons.add(termlog.TermLog())
         self.addons.add(*addons.default_addons())
