@@ -7,10 +7,18 @@ set -o nounset
 
 MITMPROXY_PATH="/home/mitmproxy/.mitmproxy"
 
+if [ -f "$MITMPROXY_PATH/mitmproxy-ca.pem" ]; then
+  f="$MITMPROXY_PATH/mitmproxy-ca.pem"
+else
+  f="$MITMPROXY_PATH"
+fi
+usermod -o \
+    -u $(stat -c "%u" "$f") \
+    -g $(stat -c "%g" "$f") \
+    mitmproxy
+
 if [[ "$1" = "mitmdump" || "$1" = "mitmproxy" || "$1" = "mitmweb" ]]; then
-  mkdir -p "$MITMPROXY_PATH"
-  chown -R mitmproxy:mitmproxy "$MITMPROXY_PATH"
-  gosu mitmproxy "$@"
+  exec gosu mitmproxy "$@"
 else
   exec "$@"
 fi
