@@ -44,7 +44,7 @@ async def noresponseapp(scope, receive, send):
     return
 
 
-async def test_asgi_full():
+async def test_asgi_full(caplog):
     ps = Proxyserver()
     addons = [
         asgiapp.WSGIApp(tapp, "testapp", 80),
@@ -54,9 +54,8 @@ async def test_asgi_full():
     with taddons.context(ps, *addons) as tctx:
         tctx.master.addons.add(next_layer.NextLayer())
         tctx.configure(ps, listen_host="127.0.0.1", listen_port=0)
-        await ps.running()
-        await tctx.master.await_log("Proxy server listening", level="info")
-        proxy_addr = ps.tcp_server.sockets[0].getsockname()[:2]
+        assert await ps.setup_servers()
+        proxy_addr = ("127.0.0.1", ps.listen_addrs()[0][1])
 
         reader, writer = await asyncio.open_connection(*proxy_addr)
         req = f"GET http://testapp:80/ HTTP/1.1\r\n\r\n"
@@ -65,6 +64,8 @@ async def test_asgi_full():
         assert header.startswith(b"HTTP/1.1 200 OK")
         body = await reader.readuntil(b"testapp")
         assert body == b"testapp"
+        writer.close()
+        await writer.wait_closed()
 
         reader, writer = await asyncio.open_connection(*proxy_addr)
         req = f"GET http://testapp:80/parameters?param1=1&param2=2 HTTP/1.1\r\n\r\n"
@@ -73,6 +74,8 @@ async def test_asgi_full():
         assert header.startswith(b"HTTP/1.1 200 OK")
         body = await reader.readuntil(b"}")
         assert body == b'{"param1": "1", "param2": "2"}'
+        writer.close()
+        await writer.wait_closed()
 
         reader, writer = await asyncio.open_connection(*proxy_addr)
         req = f"POST http://testapp:80/requestbody HTTP/1.1\r\nContent-Length: 6\r\n\r\nHello!"
@@ -81,6 +84,8 @@ async def test_asgi_full():
         assert header.startswith(b"HTTP/1.1 200 OK")
         body = await reader.readuntil(b"}")
         assert body == b'{"body": "Hello!"}'
+        writer.close()
+        await writer.wait_closed()
 
         reader, writer = await asyncio.open_connection(*proxy_addr)
         req = f"GET http://errapp:80/?foo=bar HTTP/1.1\r\n\r\n"
@@ -89,6 +94,9 @@ async def test_asgi_full():
         assert header.startswith(b"HTTP/1.1 500")
         body = await reader.readuntil(b"ASGI Error")
         assert body == b"ASGI Error"
+        writer.close()
+        await writer.wait_closed()
+        assert "ValueError" in caplog.text
 
         reader, writer = await asyncio.open_connection(*proxy_addr)
         req = f"GET http://noresponseapp:80/ HTTP/1.1\r\n\r\n"
@@ -97,3 +105,9 @@ async def test_asgi_full():
         assert header.startswith(b"HTTP/1.1 500")
         body = await reader.readuntil(b"ASGI Error")
         assert body == b"ASGI Error"
+        writer.close()
+        await writer.wait_closed()
+        assert "no response sent" in caplog.text
+
+        tctx.configure(ps, server=False)
+        assert await ps.setup_servers()
