@@ -1,12 +1,14 @@
+from unittest import mock
+
+import pyperclip
+import pytest
+
+from mitmproxy import certs
+from mitmproxy import exceptions
 from mitmproxy.addons import cut
 from mitmproxy.addons import view
-from mitmproxy import exceptions
-from mitmproxy import certs
 from mitmproxy.test import taddons
 from mitmproxy.test import tflow
-import pytest
-import pyperclip
-from unittest import mock
 
 
 def test_extract(tdata):
@@ -72,7 +74,7 @@ def qr(f):
         return fp.read()
 
 
-async def test_cut_clip():
+async def test_cut_clip(caplog):
     v = view.View()
     c = cut.Cut()
     with taddons.context() as tctx:
@@ -97,7 +99,7 @@ async def test_cut_clip():
             )
             pc.side_effect = pyperclip.PyperclipException(log_message)
             tctx.command(c.clip, "@all", "request.method")
-            await tctx.master.await_log(log_message, level="error")
+            assert log_message in caplog.text
 
 
 def test_cut_save(tmpdir):
@@ -130,7 +132,7 @@ def test_cut_save(tmpdir):
         (FileNotFoundError, "No such file or directory"),
     ],
 )
-async def test_cut_save_open(exception, log_message, tmpdir):
+async def test_cut_save_open(exception, log_message, tmpdir, caplog):
     f = str(tmpdir.join("path"))
     v = view.View()
     c = cut.Cut()
@@ -141,7 +143,7 @@ async def test_cut_save_open(exception, log_message, tmpdir):
         with mock.patch("mitmproxy.addons.cut.open") as m:
             m.side_effect = exception(log_message)
             tctx.command(c.save, "@all", "request.method", f)
-            await tctx.master.await_log(log_message, level="error")
+            assert log_message in caplog.text
 
 
 def test_cut():
@@ -171,8 +173,9 @@ def test_cut():
         assert c.cut(tflows, ["response.reason"]) == [[""]]
         assert c.cut(tflows, ["response.header[key]"]) == [[""]]
 
-    c = cut.Cut()
-    with taddons.context():
-        tflows = [tflow.ttcpflow()]
-        assert c.cut(tflows, ["request.method"]) == [[""]]
-        assert c.cut(tflows, ["response.status"]) == [[""]]
+    for f in (tflow.ttcpflow(), tflow.tudpflow()):
+        c = cut.Cut()
+        with taddons.context():
+            tflows = [f]
+            assert c.cut(tflows, ["request.method"]) == [[""]]
+            assert c.cut(tflows, ["response.status"]) == [[""]]
