@@ -7,6 +7,7 @@ import pytest
 from mitmproxy import exceptions
 from mitmproxy.addons import dumper
 from mitmproxy.http import Headers
+from mitmproxy.net.dns import response_codes
 from mitmproxy.test import taddons
 from mitmproxy.test import tflow
 from mitmproxy.test import tutils
@@ -173,7 +174,8 @@ def test_echo_request_line():
         sio.truncate(0)
 
 
-async def test_contentview():
+async def test_contentview(caplog):
+    caplog.set_level("DEBUG")
     with mock.patch("mitmproxy.contentviews.auto.ViewAuto.__call__") as va:
         va.side_effect = ValueError("")
         sio = io.StringIO()
@@ -181,7 +183,7 @@ async def test_contentview():
         with taddons.context(d) as tctx:
             tctx.configure(d, flow_detail=4)
             d.response(tflow.tflow())
-            await tctx.master.await_log("content viewer failed")
+            assert "content viewer failed" in caplog.text
 
 
 def test_tcp():
@@ -199,6 +201,21 @@ def test_tcp():
         assert "Error in TCP" in sio.getvalue()
 
 
+def test_udp():
+    sio = io.StringIO()
+    d = dumper.Dumper(sio)
+    with taddons.context(d) as ctx:
+        ctx.configure(d, flow_detail=3, showhost=True)
+        f = tflow.tudpflow()
+        d.udp_message(f)
+        assert "it's me" in sio.getvalue()
+        sio.truncate(0)
+
+        f = tflow.tudpflow(client_conn=True, err=True)
+        d.udp_error(f)
+        assert "Error in UDP" in sio.getvalue()
+
+
 def test_dns():
     sio = io.StringIO()
     d = dumper.Dumper(sio)
@@ -210,6 +227,12 @@ def test_dns():
         assert "8.8.8.8" in sio.getvalue()
         sio.truncate(0)
 
+        f = tflow.tdnsflow()
+        f.response = f.request.fail(response_codes.NOTIMP)
+        d.dns_response(f)
+        assert "NOTIMP" in sio.getvalue()
+        sio.truncate(0)
+
         f = tflow.tdnsflow(err=True)
         d.dns_error(f)
         assert "error" in sio.getvalue()
@@ -263,6 +286,27 @@ def test_http2():
         assert "HTTP/2.0 200 OK" in sio.getvalue()
 
 
+def test_quic():
+    sio = io.StringIO()
+    d = dumper.Dumper(sio)
+    with taddons.context(d):
+        f = tflow.ttcpflow()
+        f.client_conn.tls_version = "QUIC"
+        # TODO: This should not be metadata, this should be typed attributes.
+        f.metadata["quic_stream_id_client"] = 1
+        f.metadata["quic_stream_id_server"] = 1
+        d.tcp_message(f)
+        assert "quic stream 1" in sio.getvalue()
+
+        f2 = tflow.tudpflow()
+        f2.client_conn.tls_version = "QUIC"
+        # TODO: This should not be metadata, this should be typed attributes.
+        f2.metadata["quic_stream_id_client"] = 1
+        f2.metadata["quic_stream_id_server"] = 1
+        d.udp_message(f2)
+        assert "quic stream 1" in sio.getvalue()
+
+
 def test_styling():
     sio = io.StringIO()
 
