@@ -1,15 +1,15 @@
 import os
 import shlex
+from unittest import mock
 
-import pytest
 import pyperclip
+import pytest
 
 from mitmproxy import exceptions
 from mitmproxy.addons import export  # heh
+from mitmproxy.test import taddons
 from mitmproxy.test import tflow
 from mitmproxy.test import tutils
-from mitmproxy.test import taddons
-from unittest import mock
 
 
 @pytest.fixture
@@ -53,6 +53,11 @@ def tcp_flow():
     return tflow.ttcpflow()
 
 
+@pytest.fixture
+def udp_flow():
+    return tflow.tudpflow()
+
+
 @pytest.fixture(scope="module")
 def export_curl():
     e = export.Export()
@@ -88,6 +93,10 @@ class TestExportCurlCommand:
         with pytest.raises(exceptions.CommandError):
             export_curl(tcp_flow)
 
+    def test_udp(self, export_curl, udp_flow):
+        with pytest.raises(exceptions.CommandError):
+            export_curl(udp_flow)
+
     def test_escape_single_quotes_in_body(self, export_curl):
         request = tflow.tflow(
             req=tutils.treq(method=b"POST", headers=(), content=b"'&#")
@@ -153,6 +162,10 @@ class TestExportHttpieCommand:
         with pytest.raises(exceptions.CommandError):
             export.httpie_command(tcp_flow)
 
+    def test_udp(self, udp_flow):
+        with pytest.raises(exceptions.CommandError):
+            export.httpie_command(udp_flow)
+
     def test_escape_single_quotes_in_body(self):
         request = tflow.tflow(
             req=tutils.treq(method=b"POST", headers=(), content=b"'&#")
@@ -197,6 +210,13 @@ class TestRaw:
         ):
             export.raw(tcp_flow)
 
+    def test_udp(self, udp_flow):
+        with pytest.raises(
+            exceptions.CommandError,
+            match="Can't export flow with no request or response",
+        ):
+            export.raw(udp_flow)
+
 
 class TestRawRequest:
     def test_get(self, get_request):
@@ -212,6 +232,10 @@ class TestRawRequest:
         with pytest.raises(exceptions.CommandError):
             export.raw_request(tcp_flow)
 
+    def test_udp(self, udp_flow):
+        with pytest.raises(exceptions.CommandError):
+            export.raw_request(udp_flow)
+
 
 class TestRawResponse:
     def test_get(self, get_response):
@@ -226,6 +250,10 @@ class TestRawResponse:
         with pytest.raises(exceptions.CommandError):
             export.raw_response(tcp_flow)
 
+    def test_udp(self, udp_flow):
+        with pytest.raises(exceptions.CommandError):
+            export.raw_response(udp_flow)
+
 
 def qr(f):
     with open(f, "rb") as fp:
@@ -267,17 +295,16 @@ def test_export(tmp_path) -> None:
         (FileNotFoundError, "No such file or directory"),
     ],
 )
-async def test_export_open(exception, log_message, tmpdir):
+async def test_export_open(exception, log_message, tmpdir, caplog):
     f = str(tmpdir.join("path"))
     e = export.Export()
-    with taddons.context() as tctx:
-        with mock.patch("mitmproxy.addons.export.open") as m:
-            m.side_effect = exception(log_message)
-            e.file("raw_request", tflow.tflow(resp=True), f)
-            await tctx.master.await_log(log_message, level="error")
+    with mock.patch("mitmproxy.addons.export.open") as m:
+        m.side_effect = exception(log_message)
+        e.file("raw_request", tflow.tflow(resp=True), f)
+        assert log_message in caplog.text
 
 
-async def test_clip(tmpdir):
+async def test_clip(tmpdir, caplog):
     e = export.Export()
     with taddons.context() as tctx:
         tctx.configure(e)
@@ -307,4 +334,4 @@ async def test_clip(tmpdir):
             )
             pc.side_effect = pyperclip.PyperclipException(log_message)
             e.clip("raw_request", tflow.tflow(resp=True))
-            await tctx.master.await_log(log_message, level="error")
+            assert log_message in caplog.text
