@@ -1,7 +1,7 @@
 import pytest
 
-from mitmproxy.addons import intercept
 from mitmproxy import exceptions
+from mitmproxy.addons import intercept
 from mitmproxy.test import taddons
 from mitmproxy.test import tflow
 
@@ -75,3 +75,17 @@ async def test_tcp():
         f = tflow.ttcpflow()
         await tctx.cycle(r, f)
         assert not f.intercepted
+
+
+async def test_udp():
+    r = intercept.Intercept()
+    with taddons.context(r) as tctx:
+        tctx.configure(r, intercept="~udp")
+        f = tflow.tudpflow()
+        await tctx.cycle(r, f)
+        assert f.intercepted
+
+        tctx.configure(r, intercept_active=False)
+        f = tflow.tudpflow()
+        await tctx.cycle(r, f)
+        assert not f.intercepted
