@@ -1,6 +1,7 @@
 import pytest
 
-from mitmproxy.addons.modifyheaders import parse_modify_spec, ModifyHeaders
+from mitmproxy.addons.modifyheaders import ModifyHeaders
+from mitmproxy.addons.modifyheaders import parse_modify_spec
 from mitmproxy.test import taddons
 from mitmproxy.test import tflow
 from mitmproxy.test.tutils import tresp
@@ -127,7 +128,7 @@ class TestModifyHeadersFile:
             mh.request(f)
             assert f.request.headers["one"] == "two"
 
-    async def test_nonexistent(self, tmpdir):
+    async def test_nonexistent(self, tmpdir, caplog):
         mh = ModifyHeaders()
         with taddons.context(mh) as tctx:
             with pytest.raises(
@@ -142,4 +143,4 @@ class TestModifyHeadersFile:
             f = tflow.tflow()
             f.request.content = b"foo"
             mh.request(f)
-            await tctx.master.await_log("could not read")
+            assert "Could not read" in caplog.text
