@@ -1,8 +1,8 @@
 import asyncio
 import io
+from unittest import mock
 
 import pytest
-from unittest import mock
 
 import mitmproxy.io
 from mitmproxy import exceptions
@@ -47,7 +47,7 @@ class TestReadFile:
                 tctx.configure(rf, readfile_filter="~~")
             tctx.configure(rf, readfile_filter="")
 
-    async def test_read(self, tmpdir, data, corrupt_data):
+    async def test_read(self, tmpdir, data, corrupt_data, caplog_async):
         rf = readfile.ReadFile()
         with taddons.context(rf) as tctx:
             assert not rf.reading()
@@ -65,25 +65,24 @@ class TestReadFile:
             tf.write(corrupt_data.getvalue())
             tctx.configure(rf, rfile=str(tf))
             rf.running()
-            await tctx.master.await_log("corrupted")
+            await caplog_async.await_log("corrupted")
 
-    async def test_corrupt(self, corrupt_data):
+    async def test_corrupt(self, corrupt_data, caplog_async):
         rf = readfile.ReadFile()
-        with taddons.context(rf) as tctx:
+        with taddons.context(rf):
             with pytest.raises(exceptions.FlowReadException):
                 await rf.load_flows(io.BytesIO(b"qibble"))
 
-            tctx.master.clear()
+            caplog_async.clear()
             with pytest.raises(exceptions.FlowReadException):
                 await rf.load_flows(corrupt_data)
-            await tctx.master.await_log("file corrupted")
+            await caplog_async.await_log("file corrupted")
 
-    async def test_nonexistent_file(self):
+    async def test_nonexistent_file(self, caplog):
         rf = readfile.ReadFile()
-        with taddons.context(rf) as tctx:
-            with pytest.raises(exceptions.FlowReadException):
-                await rf.load_flows_from_path("nonexistent")
-            await tctx.master.await_log("nonexistent")
+        with pytest.raises(exceptions.FlowReadException):
+            await rf.load_flows_from_path("nonexistent")
+        assert "nonexistent" in caplog.text
 
 
 class TestReadFileStdin:
