@@ -1,4 +1,4 @@
-from mitmproxy import ctx
+import logging
 
 event_log = []
 
@@ -9,7 +9,7 @@ class Addon:
         return event_log
 
     def load(self, opts):
-        ctx.log.info("addon running")
+        logging.info("addon running")
         event_log.append("addonload")
 
     def configure(self, updated):
