@@ -3,9 +3,9 @@ import os
 import pytest
 from hypothesis import settings
 
-from mitmproxy import connection, options
+from mitmproxy import connection
+from mitmproxy import options
 from mitmproxy.addons.proxyserver import Proxyserver
-from mitmproxy.addons.termlog import TermLog
 from mitmproxy.proxy import context
 
 
@@ -13,9 +13,14 @@ from mitmproxy.proxy import context
 def tctx() -> context.Context:
     opts = options.Options()
     Proxyserver().load(opts)
-    TermLog().load(opts)
     return context.Context(
-        connection.Client(("client", 1234), ("127.0.0.1", 8080), 1605699329), opts
+        connection.Client(
+            peername=("client", 1234),
+            sockname=("127.0.0.1", 8080),
+            timestamp_start=1605699329,
+            state=connection.ConnectionState.OPEN,
+        ),
+        opts,
     )
 
 
