@@ -1,13 +1,10 @@
 import pytest
 
-from mitmproxy import options
-from mitmproxy.tools.console import statusbar, master
+from mitmproxy.tools.console import statusbar
 
 
-async def test_statusbar(monkeypatch):
-    o = options.Options()
-    m = master.ConsoleMaster(o)
-    m.options.update(
+async def test_statusbar(console, monkeypatch):
+    console.options.update(
         modify_headers=[":~q:foo:bar"],
         modify_body=[":~q:foo:bar"],
         ignore_hosts=["example.com", "example.org"],
@@ -24,29 +21,28 @@ async def test_statusbar(monkeypatch):
         server_replay_kill_extra=True,
         upstream_cert=False,
         stream_large_bodies="3m",
-        mode="transparent",
+        mode=["transparent"],
     )
-
-    m.options.update(view_order="url", console_focus_follow=True)
-    monkeypatch.setattr(m.addons.get("clientplayback"), "count", lambda: 42)
-    monkeypatch.setattr(m.addons.get("serverplayback"), "count", lambda: 42)
+    console.options.update(view_order="url", console_focus_follow=True)
+    monkeypatch.setattr(console.addons.get("clientplayback"), "count", lambda: 42)
+    monkeypatch.setattr(console.addons.get("serverplayback"), "count", lambda: 42)
     monkeypatch.setattr(statusbar.StatusBar, "refresh", lambda x: None)
 
-    bar = statusbar.StatusBar(m)  # this already causes a redraw
+    bar = statusbar.StatusBar(console)  # this already causes a redraw
     assert bar.ib._w
 
 
 @pytest.mark.parametrize(
     "message,ready_message",
     [
-        ("", [(None, ""), ("warn", "")]),
+        ("", [("", ""), ("warn", "")]),
         (
             ("info", "Line fits into statusbar"),
             [("info", "Line fits into statusbar"), ("warn", "")],
         ),
         (
             "Line doesn't fit into statusbar",
-            [(None, "Line doesn'\u2026"), ("warn", "(more in eventlog)")],
+            [("", "Line doesn'\u2026"), ("warn", "(more in eventlog)")],
         ),
         (
             ("alert", "Two lines.\nFirst fits"),
@@ -54,14 +50,14 @@ async def test_statusbar(monkeypatch):
         ),
         (
             "Two long lines\nFirst doesn't fit",
-            [(None, "Two long li\u2026"), ("warn", "(more in eventlog)")],
+            [("", "Two long li\u2026"), ("warn", "(more in eventlog)")],
         ),
     ],
 )
 def test_shorten_message(message, ready_message):
-    assert statusbar.ActionBar.shorten_message(message, max_width=30) == ready_message
+    assert statusbar.shorten_message(message, max_width=30) == ready_message
 
 
 def test_shorten_message_narrow():
-    shorten_msg = statusbar.ActionBar.shorten_message("error", max_width=4)
-    assert shorten_msg == [(None, "\u2026"), ("warn", "(more in eventlog)")]
+    shorten_msg = statusbar.shorten_message("error", max_width=4)
+    assert shorten_msg == [("", "\u2026"), ("warn", "(more in eventlog)")]
