@@ -1,12 +1,10 @@
-import * as React from "react"
-import {render, screen} from "../test-utils";
+import * as React from "react";
+import { render, screen } from "../test-utils";
 import Header from "../../components/Header";
-import {fireEvent} from "@testing-library/react";
-
+import { fireEvent } from "@testing-library/react";
 
 test("Header", async () => {
-
-    const {asFragment} = render(<Header/>);
+    const { asFragment } = render(<Header />);
     expect(asFragment()).toMatchSnapshot();
 
     fireEvent.click(screen.getByText("Options"));
@@ -18,5 +16,5 @@ test("Header", async () => {
     expect(screen.getByText("Open...")).toBeTruthy();
 
     fireEvent.click(screen.getByText("File"));
-    expect(screen.queryByText("Open...")).toBeNull()
+    expect(screen.queryByText("Open...")).toBeNull();
 });
